#!/usr/bin/env python
# encoding: utf-8

import os
import unittest
from datetime import timedelta

from zijin.zijin_data import get_market_data
from zijin.zijin_config import START_DATE, END_DATE
from zijin.zijin_functions import Strategy
from zijin.zijin_basic import strategy_run


class TestCase(unittest.TestCase):
    # def test1(self):
    #     get_market_data(START_DATE, END_DATE)
    #
    # def test2(self):
    #     strategy = Strategy(None, None, None)
    #     strategy.get_market_data(END_DATE - timedelta(days=1))

    def test3(self):
        predict_format = [
            {'name': '预测操作', 'type': 'string', 'information': '预测走势'},
            {'name': '正确率', 'type': 'float', 'information': '统计预测准确率'}
        ]

        strategy_run('/home/xxh/workspace/zijin/test', 'predict.py', 'predict', predict_format)

if __name__ == '__main__':
    unittest.main()


#!/usr/bin/env python
# encoding: utf-8


from collections import defaultdict
from datetime import datetime
import csv
import inspect
import os

from zijin_objects import Bar


def line_to_bar(line_content):
    """
    change csv line content to object bar
    :param line_content: string 
    :return: 
    """
    cell_array = line_content
    trade_datetime = datetime.strptime(cell_array[1], '%Y-%m-%d %H:%M:%S.%f')
    trade_date = trade_datetime.date()
    open_price = float(cell_array[2])
    high_price = float(cell_array[3])
    low_price = float(cell_array[4])
    close_price = float(cell_array[5])
    volume = float(cell_array[6])
    kargs = {'open': open_price, 'high': high_price, 'low': low_price,
             'close': close_price, 'datetime': trade_datetime, 'volume': volume,
             'trade_date': trade_date}
    return Bar(**kargs)


# the data.csv is provided automatically
def get_market_data(start_date, end_date):
    print 'loading data .....'
    dir_name = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
    path = os.path.join(dir_name, 'data.csv')
    market_data = defaultdict(list)
    with open(path, 'r') as csv_file:
        content = csv.reader(csv_file)
        next(content, None)
        for line in content:
            bar = line_to_bar(line)
            market_data[bar.trade_date].append(bar)
    return {k: v for k, v in market_data.iteritems() if start_date <= k < end_date}


#!/usr/bin/env python
# encoding: utf-8

import os
import shutil
from datetime import datetime, date

from zijin_functions import StrategyManager
from zijin_config import ORDER_DIR, LOG_DIR, PREDICT_DIR, START_DATE, END_DATE


def strategy_run(work_dir, script_path, script_mode, predict_format, start=START_DATE, end=END_DATE):
    """
    :param work_dir: 工作目录，脚本和模型文件所在目录
    :param script_path: 脚本名称
    :param script_mode: 脚本类型， predict or trade
    :param predict_format: 如果是预测脚本，预测的格式，如果是交易脚本，None
    :param start: 回测开始时间（选填）
    :param end: 回测结束时间（选填）
    :return: 
    """
    os.chdir(work_dir)
    if os.path.exists(LOG_DIR):
        shutil.rmtree(LOG_DIR)
    os.makedirs(LOG_DIR)
    if os.path.exists(PREDICT_DIR):
        shutil.rmtree(PREDICT_DIR)
    if os.path.exists(ORDER_DIR):
        shutil.rmtree(ORDER_DIR)
    if script_mode == 'predict':
        if isinstance(predict_format, list):
            for arg in predict_format:
                if not isinstance(arg, dict):
                    raise Exception('exception: single predict format should be dict')
        else:
            raise Exception('exception: predict format should be list')
        os.makedirs(PREDICT_DIR)
    else:
        os.makedirs(ORDER_DIR)
    strategy_manager = StrategyManager()
    if isinstance(start, datetime):
        start = start.date()
    if isinstance(end, datetime):
        end = end.date()
    if not isinstance(start, date):
        raise Exception('start should be datetime or date')
    if not isinstance(end, date):
        raise Exception('end should be datetime or date')
    strategy_manager.run_strategy(script_path[:-3], script_mode, predict_format, start, end)


#!/usr/bin/env python
# encoding: utf-8
from datetime import datetime, timedelta, date
import os

from zijin_objects import Predict, Order
from zijin_util import error_decorator, date2str
from zijin_log import zijin_log, zijin_order, zijin_predict
from zijin_data import get_market_data
from zijin_config import LOG_PREFIX, LOG_DIR, ORDER_PREFIX, ORDER_DIR, LOG_SUFFIX, ORDER_SUFFIX, \
    PREDICT_DIR, PREDICT_PREFIX, PREDICT_SUFFIX


class SimpleResource(object):
    def __init__(self, id_=None):
        self.id = id_


class Strategy(object):
    callback_function_names = ['on_init', 'on_start', 'on_stop', 'on_tick', 'on_bar', 'on_order', 'on_newday']
    provide_function_names = ['buy', 'sell', 'short', 'cover', 'log', 'predict', 'get_time', 'get_market_data']

    def __init__(self, user_script, script_mode, predict_format):
        context = SimpleResource()
        function = SimpleResource()
        var = SimpleResource()
        context.var = var
        context.function = function
        self._datetime = None
        self._trade_date = None
        self._context = context
        self._script = user_script
        self._script_mode = script_mode
        if self._script_mode == 'predict':
            self._predict_format = predict_format

        for name in self.callback_function_names:
            setattr(self, name, error_decorator(getattr(self, name)))
        for name in self.provide_function_names:
            setattr(function, name, getattr(self, name))

    def on_init(self):
        self._script.on_init(self._context)

    def on_start(self):
        self._script.on_start(self._context)

    def on_stop(self):
        self._script.on_stop(self._context)

    def on_bar(self, bar):
        self._context.var.bar = bar
        self._script.on_bar(self._context)

    def on_tick(self, tick):
        self._context.var.tick = tick
        self._script.on_tick(self._context)

    def on_order(self, order):
        print 'using on_order......'
        self._script.on_order(self._context)

    def on_newday(self):
        self._script.on_newday(self._context)

    def datetime_update(self, _datetime, trade_date):
        self._datetime = _datetime
        self._trade_date = trade_date

    def buy(self, price, volume=1):
        self.send_order('buy', price, volume)

    def sell(self, price, volume=1):
        self.send_order('sell', price, volume)

    def short(self, price, volume=1):
        self.send_order('sell', price, volume)

    def cover(self, price, volume=1):
        self.send_order('buy', price, volume)

    def send_order(self, type, price, volume):
        _datetime, trade_date = self.get_time()
        arg_dict = {'trade_date': trade_date, 'price': price, 'type': type, 'volume': volume,
                    'datetime': _datetime}
        order = Order(**arg_dict)
        zijin_order(os.path.join(ORDER_DIR, ORDER_PREFIX + date2str(trade_date) + ORDER_SUFFIX), order)

    def get_market_data(self, trade_date, type='bar'):
        if isinstance(trade_date, date):
            try:
                bars = get_market_data(start_date=trade_date, end_date=trade_date + timedelta(days=1))[trade_date]
            except KeyError:
                bars = []
        elif isinstance(trade_date, datetime):
            try:
                trade_date = datetime.date()
                bars =  get_market_data(start_date=trade_date, end_date=trade_date + timedelta(days=1))[trade_date]
            except KeyError:
                bars = []
        else:
            bars = []
        return [vars(bar) for bar in bars]

    def get_time(self):
        if self._datetime is None:
            now_datetime = datetime.now()
            self._datetime = now_datetime
        else:
            now_datetime = self._datetime

        if self._trade_date is None:
            trade_date = self._datetime.date()
        else:
            trade_date = self._trade_date
        return now_datetime, trade_date

    def log(self, content):
        now_datetime, trade_date = self.get_time()
        if isinstance(content, unicode):
            content = content.encode('utf-8')
        elif not isinstance(content, str):
            content = repr(content)
        zijin_log(os.path.join(LOG_DIR, LOG_PREFIX + date2str(trade_date) + LOG_SUFFIX), content + '\r\n')

    def predict(self, data):
        if self._script_mode != 'predict':
            raise Exception('use predict function in trade strategy')
        if not isinstance(data, dict):
            raise Exception('predict data must be dict')
        now_datetime, trade_date = self.get_time()
        data['datetime'] = now_datetime
        predict = Predict(**data)
        is_valid, error = predict.is_valid(self._predict_format)
        if not is_valid:
            raise Exception(error)
        zijin_predict(os.path.join(PREDICT_DIR, PREDICT_PREFIX + date2str(trade_date) + PREDICT_SUFFIX), vars(predict))


class StrategyManager(object):
    def __init__(self):
        super(StrategyManager, self).__init__()

    def run_strategy(self, script_path, script_mode, predict_format, start_date, end_date):
        script = None
        exec 'import {:s} as script'.format(script_path)
        strategy = Strategy(script, script_mode, predict_format)
        for function_name in strategy.callback_function_names:
            if not hasattr(script, function_name):
                raise Exception('{:s} function missing'.format(function_name))
        start = False
        first_day = True
        market_date = get_market_data(start_date=start_date, end_date=end_date)
        current_date = start_date
        print 'run strategy.....'
        strategy.on_init()
        while current_date < end_date:
            try:
                bars = market_date[current_date]
            except KeyError:
                current_date = current_date + timedelta(days=1)
                continue
            print 'date {:s} start'.format(date2str(current_date))
            strategy.datetime_update(datetime.combine(current_date, datetime.min.time()), current_date)
            if not start:
                strategy.on_start()
                start = True
            if first_day:
                first_day = False
            else:
                strategy.on_newday()
            for bar in bars:
                strategy.datetime_update(bar.datetime, current_date)
                strategy.on_bar(bar)
            print 'date {:s} end'.format(date2str(current_date))
            current_date = current_date + timedelta(days=1)


from datetime import datetime

LOG_DIR = 'logs'
LOG_PREFIX = 'log'
LOG_SUFFIX = '.txt'
ORDER_DIR = 'orders'
ORDER_PREFIX = 'order'
ORDER_SUFFIX = '.csv'
PREDICT_DIR = 'predicts'
PREDICT_PREFIX = 'predict'
PREDICT_SUFFIX = '.csv'
START_DATE = datetime.strptime('2017-05-22', '%Y-%m-%d').date()
END_DATE = datetime.strptime('2017-06-01', '%Y-%m-%d').date()
STRATEGY_TIMEOUT = 50
ORDER_HEADER = ['datetime', 'type', 'price', 'volume']


#!/usr/bin/env python
# encoding: utf-8

from zijin_util import datetime2str

PREDICT_TYPE_MAP = {'int': int, 'float': float, 'string': str, 'bool': bool}


class Bar(object):
    """
    required attributes
    open: float
    high: float
    low: float
    close: float
    datetime: datetime
    trade_date: date
    volume: float
    """
    def __init__(self, **kargs):
        required_fields = ['open', 'high', 'low', 'close', 'datetime', 'trade_date', 'volume']
        for key in kargs:
            if key in required_fields:
                required_fields.remove(key)
        if len(required_fields) == 0:
            self.__dict__ = kargs
        else:
            raise Exception('fileds {:s} missing'.format(','.join(required_fields)))


class Order(object):
    """
    required attributes
    trade_date: date
    price: float
    trade_volume: float
    type: string
    datetime: datetime
    """
    def __init__(self, **kargs):
        required_fields = ['trade_date', 'price', 'volume', 'type', 'datetime']
        for key in kargs:
            if key in required_fields:
                required_fields.remove(key)
        if len(required_fields) == 0:
            self.__dict__ = kargs
        else:
            raise Exception('fields {:s} missing'.format(','.join(required_fields)))

    def __repr__(self):
        return ','.join([datetime2str(self.datetime), str(self.type), str(self.price), str(self.volume)])


class Predict(object):
    """
    required attributes
    datetime: datetime
    """
    def __init__(self, **kargs):
        required_fields = ['datetime']
        for key in kargs:
            if key in required_fields:
                required_fields.remove(key)
        if len(required_fields) == 0:
            self.__dict__ = kargs
        else:
            raise Exception('fields {:s} missing'.format(','.join(required_fields)))

    def is_valid(self, predict_format):
        for item in predict_format:
            value = getattr(self, item['name'], None)
            if value is None or not isinstance(value, PREDICT_TYPE_MAP[item['type']]):
                return False, 'predict data not match format: {:s} {:s}'.format(item['name'], item['type'])
            return True, ''






#!/usr/bin/env python
# encoding: utf-8

import contextlib
import signal

from zijin_config import STRATEGY_TIMEOUT


def handle_timeout(sig, frame):
    raise Exception('timeout exception, runtime more than {:d}'.format(STRATEGY_TIMEOUT))


@contextlib.contextmanager
def timeout(seconds):
    signal.signal(signal.SIGALRM, handle_timeout)
    signal.alarm(seconds)
    yield


def error_decorator(func):
    def wrapper(*args, **kargs):
        if func.__name__ in ['on_newday', 'on_init']:
            return func(*args, **kargs)
        with timeout(STRATEGY_TIMEOUT):
            return func(*args, **kargs)
    return wrapper


def datetime2str(_datetime):
    return _datetime.strftime('%Y-%m-%d %H:%M:%S')


def date2str(_date):
    return _date.strftime('%Y-%m-%d')




from setuptools import setup

setup(name='zijin',
      version='1.4',
      description='zijin test package',
      url='https://github.com/xxh13/zijin_.git',
      author='xxh13',
      author_email='15850781776@163.com',
      license='MIT',
      packages=['zijin'],
      install_requires=['bson'],
      zip_safe=False,
      include_package_data=True
)



#!/usr/bin/env python
# encoding: utf-8
import os
import csv

from zijin_config import ORDER_HEADER


def zijin_log(path, content):
    with open(path, 'a') as f:
        f.write(content)


def zijin_order(path, order):
    if os.path.exists(path):
        first_write = False
    else:
        first_write = True
    with open(path, 'a') as f:
        csv_writer = csv.writer(f)
        if first_write:
            csv_writer.writerow(ORDER_HEADER)
        csv_writer.writerow([order.datetime, order.type, order.price, order.volume])


def zijin_predict(path, predict):
    if os.path.exists(path):
        first_write = False
    else:
        first_write = True
    with open(path, 'a') as f:
        predict_value_list = []
        predict_value_list.append(predict['datetime'])
        keys = predict.keys()
        keys.remove('datetime')
        keys = sorted(keys)
        csv_writer = csv.writer(f)
        if first_write:
            csv_writer.writerow(['datetime'] + sorted(keys))
        for key in keys:
            predict_value_list.append(predict[key])
        csv_writer.writerow(predict_value_list)
